@@ -1,12 +1,12 @@
-/* $Id: browser.c,v 1.116 2006/12/11 22:11:32 dolorous Exp $ */
+/* $Id: browser.c,v 1.119.2.5 2007/10/11 05:01:29 dolorous Exp $ */
 /**************************************************************************
  *   browser.c                                                            *
  *                                                                        *
- *   Copyright (C) 2001, 2002, 2003, 2004 Chris Allegretta                *
- *   Copyright (C) 2005, 2006 David Lawrence Ramsey                       *
+ *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007               *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -44,7 +44,7 @@ static size_t selected = 0;
 static bool search_last_file = FALSE;
 	/* Have we gone past the last file while searching? */
 
-/* Our main file browser function.  path is the tilde-expanded path to
+/* Our main file browser function.  path is the tilde-expanded path we
  * start browsing from. */
 char *do_browser(char *path, DIR *dir)
 {
@@ -57,7 +57,7 @@ char *do_browser(char *path, DIR *dir)
 	/* The directory we were in, if any, before backing up via
 	 * browsing to "..". */
     char *ans = NULL;
-	/* The last answer the user typed on the statusbar. */
+	/* The last answer the user typed at the statusbar prompt. */
     size_t old_selected;
 	/* The selected file we had before the current selected file. */
 
@@ -131,38 +131,32 @@ char *do_browser(char *path, DIR *dir)
 		{
 		    int mouse_x, mouse_y;
 
-		    if (!get_mouseinput(&mouse_x, &mouse_y, TRUE)) {
-			/* We can click in the edit window to select a
-			 * filename. */
-			if (wenclose(edit, mouse_y, mouse_x)) {
-			    /* Subtract out the size of topwin. */
-			    mouse_y -= 2 - no_more_space();
-
-			    /* longest is the width of each column.
-			     * There are two spaces between each
-			     * column. */
-			    selected = (fileline / editwinrows) *
+		    /* We can click on the edit window to select a
+		     * filename. */
+		    if (get_mouseinput(&mouse_x, &mouse_y, TRUE) == 0 &&
+			wmouse_trafo(edit, &mouse_y, &mouse_x, FALSE)) {
+			/* longest is the width of each column.  There
+			 * are two spaces between each column. */
+			selected = (fileline / editwinrows) *
 				(editwinrows * width) + (mouse_y *
 				width) + (mouse_x / (longest + 2));
 
-			    /* If they clicked beyond the end of a row,
-			     * select the filename at the end of that
-			     * row. */
-			    if (mouse_x > width * (longest + 2))
-				selected--;
+			/* If they clicked beyond the end of a row,
+			 * select the filename at the end of that
+			 * row. */
+			if (mouse_x > width * (longest + 2))
+			    selected--;
 
-			    /* If we're off the screen, select the last
-			     * filename. */
-			    if (selected > filelist_len - 1)
-				selected = filelist_len - 1;
+			/* If we're off the screen, select the last
+			 * filename. */
+			if (selected > filelist_len - 1)
+			    selected = filelist_len - 1;
 
-			    /* If we selected the same filename as last
-			     * time, put back the Enter key so that it's
-			     * read in. */
-			    if (old_selected == selected)
-				unget_kbinput(NANO_ENTER_KEY, FALSE,
-					FALSE);
-			}
+			/* If we selected the same filename as last
+			 * time, put back the Enter key so that it's
+			 * read in. */
+			if (old_selected == selected)
+			    unget_kbinput(NANO_ENTER_KEY, FALSE, FALSE);
 		    }
 		}
 		break;
@@ -233,7 +227,7 @@ char *do_browser(char *path, DIR *dir)
 
 		/* If the directory begins with a newline (i.e. an
 		 * encoded null), treat it as though it's blank. */
-		if (i < 0 || answer[0] == '\n') {
+		if (i < 0 || *answer == '\n') {
 		    /* We canceled.  Indicate that on the statusbar, and
 		     * blank out ans, since we're done with it. */
 		    statusbar(_("Cancelled"));
@@ -833,8 +827,8 @@ int filesearch_init(void)
     backupstring = NULL;
 
     /* Cancel any search, or just return with no previous search. */
-    if (i == -1 || (i < 0 && last_search[0] == '\0') ||
-	    (i == 0 && answer[0] == '\0')) {
+    if (i == -1 || (i < 0 && *last_search == '\0') || (i == 0 &&
+	*answer == '\0')) {
 	statusbar(_("Cancelled"));
 	return -1;
     } else {
@@ -844,9 +838,8 @@ int filesearch_init(void)
 #ifdef HAVE_REGEX_H
 		/* Use last_search if answer is an empty string, or
 		 * answer if it isn't. */
-		if (ISSET(USE_REGEXP) &&
-			regexp_init((i == -2) ? last_search :
-			answer) == 0)
+		if (ISSET(USE_REGEXP) && !regexp_init((i == -2) ?
+			last_search : answer))
 		    return -1;
 #endif
 		break;
@@ -990,7 +983,7 @@ void do_filesearch(void)
 	return;
 
     /* If answer is now "", copy last_search into answer. */
-    if (answer[0] == '\0')
+    if (*answer == '\0')
 	answer = mallocstrcpy(answer, last_search);
     else
 	last_search = mallocstrcpy(last_search, answer);
@@ -1030,7 +1023,7 @@ void do_fileresearch(void)
     if (last_search[0] != '\0') {
 #ifdef HAVE_REGEX_H
 	/* Since answer is "", use last_search! */
-	if (ISSET(USE_REGEXP) && regexp_init(last_search) == 0)
+	if (ISSET(USE_REGEXP) && !regexp_init(last_search))
 	    return;
 #endif
 
