@@ -1,12 +1,12 @@
-/* $Id: chars.c,v 1.122.2.2 2007/04/23 14:03:28 dolorous Exp $ */
+/* $Id: chars.c,v 1.122.2.13 2007/10/11 05:01:29 dolorous Exp $ */
 /**************************************************************************
  *   chars.c                                                              *
  *                                                                        *
- *   Copyright (C) 2001, 2002, 2003, 2004 Chris Allegretta                *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006, 2007               *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -506,9 +506,12 @@ int mbstrcasecmp(const char *s1, const char *s2)
 /* This function is equivalent to strncasecmp(). */
 int nstrncasecmp(const char *s1, const char *s2, size_t n)
 {
+    if (s1 == s2)
+	return 0;
+
     assert(s1 != NULL && s2 != NULL);
 
-    for (; n > 0 && *s1 != '\0' && *s2 != '\0'; n--, s1++, s2++) {
+    for (; *s1 != '\0' && *s2 != '\0' && n > 0; s1++, s2++, n--) {
 	if (tolower(*s1) != tolower(*s2))
 	    break;
     }
@@ -526,12 +529,16 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 	char *s1_mb, *s2_mb;
 	wchar_t ws1, ws2;
 
+	if (s1 == s2)
+	    return 0;
+
 	assert(s1 != NULL && s2 != NULL);
 
 	s1_mb = charalloc(MB_CUR_MAX);
 	s2_mb = charalloc(MB_CUR_MAX);
 
-	while (n > 0 && *s1 != '\0' && *s2 != '\0') {
+	for (; *s1 != '\0' && *s2 != '\0' && n > 0; s1 +=
+		move_mbright(s1, 0), s2 += move_mbright(s2, 0), n--) {
 	    bool bad_s1_mb = FALSE, bad_s2_mb = FALSE;
 	    int s1_mb_len, s2_mb_len;
 
@@ -551,63 +558,38 @@ int mbstrncasecmp(const char *s1, const char *s2, size_t n)
 		bad_s2_mb = TRUE;
 	    }
 
-	    if (n == 0 || bad_s1_mb != bad_s2_mb ||
-		towlower(ws1) != towlower(ws2))
+	    if (bad_s1_mb != bad_s2_mb || towlower(ws1) !=
+		towlower(ws2))
 		break;
-
-	    s1 += s1_mb_len;
-	    s2 += s2_mb_len;
-	    n--;
 	}
 
 	free(s1_mb);
 	free(s2_mb);
 
-	return towlower(ws1) - towlower(ws2);
+	return (n > 0) ? towlower(ws1) - towlower(ws2) : 0;
     } else
 #endif
 	return strncasecmp(s1, s2, n);
 }
 
 #ifndef HAVE_STRCASESTR
-/* This function, nstrcasestr() (originally mutt_stristr()), was adapted
- * from mutt 1.2.4i (lib.c).  Here is the notice from that file, with
- * the Free Software Foundation's address updated:
- *
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 Michael R. Elkins
- * <me@cs.hmc.edu>
- * Copyright (C) 1999, 2000 Thomas Roessler <roessler@guug.de>
- *
- *     This program is free software; you can redistribute it
- *     and/or modify it under the terms of the GNU General Public
- *     License as published by the Free Software Foundation; either
- *     version 2 of the License, or (at your option) any later
- *     version.
- *
- *     This program is distributed in the hope that it will be
- *     useful, but WITHOUT ANY WARRANTY; without even the implied
- *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *     PURPOSE.  See the GNU General Public License for more
- *     details.
- *
- *     You should have received a copy of the GNU General Public
- *     License along with this program; if not, write to the Free
- *     Software Foundation, Inc., 51 Franklin St, Fifth Floor,
- *     Boston, MA  02110-1301, USA. */
-
 /* This function is equivalent to strcasestr(). */
-const char *nstrcasestr(const char *haystack, const char *needle)
+char *nstrcasestr(const char *haystack, const char *needle)
 {
+    size_t haystack_len, needle_len;
+
     assert(haystack != NULL && needle != NULL);
 
-    for (; *haystack != '\0'; haystack++) {
-	const char *r = haystack, *q = needle;
+    if (*needle == '\0')
+	return (char *)haystack;
 
-	for (; tolower(*r) == tolower(*q) && *q != '\0'; r++, q++)
-	    ;
+    haystack_len = strlen(haystack);
+    needle_len = strlen(needle);
 
-	if (*q == '\0')
-	    return haystack;
+    for (; *haystack != '\0' && haystack_len >= needle_len; haystack++,
+	haystack_len--) {
+	if (strncasecmp(haystack, needle, needle_len) == 0)
+	    return (char *)haystack;
     }
 
     return NULL;
@@ -615,62 +597,27 @@ const char *nstrcasestr(const char *haystack, const char *needle)
 #endif
 
 /* This function is equivalent to strcasestr() for multibyte strings. */
-const char *mbstrcasestr(const char *haystack, const char *needle)
+char *mbstrcasestr(const char *haystack, const char *needle)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *r_mb, *q_mb;
-	wchar_t wr, wq;
-	bool found_needle = FALSE;
+	size_t haystack_len, needle_len;
 
 	assert(haystack != NULL && needle != NULL);
 
-	r_mb = charalloc(MB_CUR_MAX);
-	q_mb = charalloc(MB_CUR_MAX);
+	if (*needle == '\0')
+	    return (char *)haystack;
 
-	while (*haystack != '\0') {
-	    const char *r = haystack, *q = needle;
-	    int r_mb_len, q_mb_len;
+	haystack_len = mbstrlen(haystack);
+	needle_len = mbstrlen(needle);
 
-	    while (*q != '\0') {
-		bool bad_r_mb = FALSE, bad_q_mb = FALSE;
-
-		r_mb_len = parse_mbchar(r, r_mb, NULL);
-
-		if (mbtowc(&wr, r_mb, r_mb_len) < 0) {
-		    mbtowc(NULL, NULL, 0);
-		    wr = (unsigned char)*r;
-		    bad_r_mb = TRUE;
-		}
-
-		q_mb_len = parse_mbchar(q, q_mb, NULL);
-
-		if (mbtowc(&wq, q_mb, q_mb_len) < 0) {
-		    mbtowc(NULL, NULL, 0);
-		    wq = (unsigned char)*q;
-		    bad_q_mb = TRUE;
-		}
-
-		if (bad_r_mb != bad_q_mb ||
-			towlower(wr) != towlower(wq))
-		    break;
-
-		r += r_mb_len;
-		q += q_mb_len;
-	    }
-
-	    if (*q == '\0') {
-		found_needle = TRUE;
-		break;
-	    }
-
-	    haystack += move_mbright(haystack, 0);
+	for (; *haystack != '\0' && haystack_len >= needle_len;
+		haystack += move_mbright(haystack, 0), haystack_len--) {
+	    if (mbstrncasecmp(haystack, needle, needle_len) == 0)
+		return (char *)haystack;
 	}
 
-	free(r_mb);
-	free(q_mb);
-
-	return found_needle ? haystack : NULL;
+	return NULL;
     } else
 #endif
 	return strcasestr(haystack, needle);
@@ -679,19 +626,27 @@ const char *mbstrcasestr(const char *haystack, const char *needle)
 #if !defined(NANO_TINY) || !defined(DISABLE_TABCOMP)
 /* This function is equivalent to strstr(), except in that it scans the
  * string in reverse, starting at rev_start. */
-const char *revstrstr(const char *haystack, const char *needle, const
-	char *rev_start)
+char *revstrstr(const char *haystack, const char *needle, const char
+	*rev_start)
 {
+    size_t rev_start_len, needle_len;
+
     assert(haystack != NULL && needle != NULL && rev_start != NULL);
 
-    for (; rev_start >= haystack; rev_start--) {
-	const char *r, *q;
+    if (*needle == '\0')
+	return (char *)rev_start;
 
-	for (r = rev_start, q = needle; *r == *q && *q != '\0'; r++, q++)
-	    ;
+    needle_len = strlen(needle);
 
-	if (*q == '\0')
-	    return rev_start;
+    if (strlen(haystack) < needle_len)
+	return NULL;
+
+    rev_start_len = strlen(rev_start);
+
+    for (; rev_start >= haystack; rev_start--, rev_start_len++) {
+	if (rev_start_len >= needle_len && strncmp(rev_start, needle,
+		needle_len) == 0)
+	    return (char *)rev_start;
     }
 
     return NULL;
@@ -701,19 +656,27 @@ const char *revstrstr(const char *haystack, const char *needle, const
 #ifndef NANO_TINY
 /* This function is equivalent to strcasestr(), except in that it scans
  * the string in reverse, starting at rev_start. */
-const char *revstrcasestr(const char *haystack, const char *needle,
-	const char *rev_start)
+char *revstrcasestr(const char *haystack, const char *needle, const char
+	*rev_start)
 {
+    size_t rev_start_len, needle_len;
+
     assert(haystack != NULL && needle != NULL && rev_start != NULL);
 
-    for (; rev_start >= haystack; rev_start--) {
-	const char *r = rev_start, *q = needle;
+    if (*needle == '\0')
+	return (char *)rev_start;
 
-	for (; tolower(*r) == tolower(*q) && *q != '\0'; r++, q++)
-	    ;
+    needle_len = strlen(needle);
 
-	if (*q == '\0')
-	    return rev_start;
+    if (strlen(haystack) < needle_len)
+	return NULL;
+
+    rev_start_len = strlen(rev_start);
+
+    for (; rev_start >= haystack; rev_start--, rev_start_len++) {
+	if (rev_start_len >= needle_len && strncasecmp(rev_start,
+		needle, needle_len) == 0)
+	    return (char *)rev_start;
     }
 
     return NULL;
@@ -722,67 +685,41 @@ const char *revstrcasestr(const char *haystack, const char *needle,
 /* This function is equivalent to strcasestr() for multibyte strings,
  * except in that it scans the string in reverse, starting at
  * rev_start. */
-const char *mbrevstrcasestr(const char *haystack, const char *needle,
-	const char *rev_start)
+char *mbrevstrcasestr(const char *haystack, const char *needle, const
+	char *rev_start)
 {
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *r_mb, *q_mb;
-	wchar_t wr, wq;
-	bool begin_line = FALSE, found_needle = FALSE;
+	bool begin_line = FALSE;
+	size_t rev_start_len, needle_len;
 
 	assert(haystack != NULL && needle != NULL && rev_start != NULL);
 
-	r_mb = charalloc(MB_CUR_MAX);
-	q_mb = charalloc(MB_CUR_MAX);
+	if (*needle == '\0')
+	    return (char *)rev_start;
+
+	needle_len = mbstrlen(needle);
+
+	if (mbstrlen(haystack) < needle_len)
+	    return NULL;
+
+	rev_start_len = mbstrlen(rev_start);
 
 	while (!begin_line) {
-	    const char *r = rev_start, *q = needle;
-	    int r_mb_len, q_mb_len;
-
-	    while (*q != '\0') {
-		bool bad_r_mb = FALSE, bad_q_mb = FALSE;
-
-		r_mb_len = parse_mbchar(r, r_mb, NULL);
-
-		if (mbtowc(&wr, r_mb, r_mb_len) < 0) {
-		    mbtowc(NULL, NULL, 0);
-		    wr = (unsigned char)*r;
-		    bad_r_mb = TRUE;
-		}
-
-		q_mb_len = parse_mbchar(q, q_mb, NULL);
-
-		if (mbtowc(&wq, q_mb, q_mb_len) < 0) {
-		    mbtowc(NULL, NULL, 0);
-		    wq = (unsigned char)*q;
-		    bad_q_mb = TRUE;
-		}
-
-		if (bad_r_mb != bad_q_mb ||
-			towlower(wr) != towlower(wq))
-		    break;
-
-		r += r_mb_len;
-		q += q_mb_len;
-	    }
-
-	    if (*q == '\0') {
-		found_needle = TRUE;
-		break;
-	    }
+	    if (rev_start_len >= needle_len && mbstrncasecmp(rev_start,
+		needle, needle_len) == 0)
+		return (char *)rev_start;
 
 	    if (rev_start == haystack)
 		begin_line = TRUE;
-	    else
+	    else {
 		rev_start = haystack + move_mbleft(haystack, rev_start -
 			haystack);
+		rev_start_len++;
+	    }
 	}
 
-	free(r_mb);
-	free(q_mb);
-
-	return found_needle ? rev_start : NULL;
+	return NULL;
     } else
 #endif
 	return revstrcasestr(haystack, needle, rev_start);
@@ -803,7 +740,7 @@ size_t nstrnlen(const char *s, size_t maxlen)
 
     assert(s != NULL);
 
-    for (; maxlen > 0 && *s != '\0'; maxlen--, n++, s++)
+    for (; *s != '\0' && maxlen > 0; s++, maxlen--, n++)
 	;
 
     return n;
@@ -818,18 +755,10 @@ size_t mbstrnlen(const char *s, size_t maxlen)
 #ifdef ENABLE_UTF8
     if (use_utf8) {
 	size_t n = 0;
-	int s_mb_len;
 
-	while (*s != '\0') {
-	    s_mb_len = parse_mbchar(s, NULL, NULL);
-
-	    if (maxlen == 0)
-		break;
-
-	    maxlen--;
-	    s += s_mb_len;
-	    n++;
-	}
+	for (; *s != '\0' && maxlen > 0; s += move_mbright(s, 0),
+		maxlen--, n++)
+	    ;
 
 	return n;
     } else
@@ -893,11 +822,9 @@ char *mbstrpbrk(const char *s, const char *accept)
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	while (*s != '\0') {
+	for (; *s != '\0'; s += move_mbright(s, 0)) {
 	    if (mbstrchr(accept, s) != NULL)
 		return (char *)s;
-
-	    s += move_mbright(s, 0);
 	}
 
 	return NULL;
@@ -979,18 +906,16 @@ bool has_blank_mbchars(const char *s)
 
 #ifdef ENABLE_UTF8
     if (use_utf8) {
-	char *chr_mb = charalloc(MB_CUR_MAX);
 	bool retval = FALSE;
+	char *chr_mb = charalloc(MB_CUR_MAX);
 
-	while (*s != '\0') {
-	    int chr_mb_len = parse_mbchar(s, chr_mb, NULL);
+	for (; *s != '\0'; s += move_mbright(s, 0)) {
+	    parse_mbchar(s, chr_mb, NULL);
 
 	    if (is_blank_mbchar(chr_mb)) {
 		retval = TRUE;
 		break;
 	    }
-
-	    s += chr_mb_len;
 	}
 
 	free(chr_mb);
