@@ -1,12 +1,12 @@
-/* $Id: files.c,v 1.348.2.18 2007/04/21 20:33:56 dolorous Exp $ */
+/* $Id: files.c,v 1.348.2.36 2007/11/17 20:34:38 dolorous Exp $ */
 /**************************************************************************
  *   files.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -382,12 +382,15 @@ void read_file(FILE *f, const char *filename)
 	 * conversion isn't disabled, handle it! */
 	if (input == '\n') {
 #ifndef NANO_TINY
-	    /* If there's a '\r' before the '\n', set format to DOS if
-	     * we currently think this is a *nix file, or to both if we
-	     * currently think it's a Mac file. */
-	    if (!ISSET(NO_CONVERT) && i > 0 && buf[i - 1] == '\r' &&
-		(format == 0 || format == 2))
-		format++;
+	    /* If it's a DOS file or a DOS/Mac file ('\r' before '\n' on
+	     * the first line if we think it's a *nix file, or on any
+	     * line otherwise), and file conversion isn't disabled,
+	     * handle it! */
+	    if (!ISSET(NO_CONVERT) && (num_lines == 0 || format != 0) &&
+		i > 0 && buf[i - 1] == '\r') {
+		if (format == 0 || format == 2)
+		    format++;
+	    }
 #endif
 
 	    /* Read in the line properly. */
@@ -401,9 +404,11 @@ void read_file(FILE *f, const char *filename)
 	    buf[0] = '\0';
 	    i = 0;
 #ifndef NANO_TINY
-	/* If it's a Mac file ('\r' without '\n'), and file conversion
-	 * isn't disabled, handle it! */
-	} else if (!ISSET(NO_CONVERT) && i > 0 && buf[i - 1] == '\r') {
+	/* If it's a Mac file ('\r' without '\n' on the first line if we
+	 * think it's a *nix file, or on any line otherwise), and file
+	 * conversion isn't disabled, handle it! */
+	} else if (!ISSET(NO_CONVERT) && (num_lines == 0 ||
+		format != 0) && i > 0 && buf[i - 1] == '\r') {
 	    /* If we currently think the file is a *nix file, set format
 	     * to Mac.  If we currently think the file is a DOS file,
 	     * set format to both DOS and Mac. */
@@ -695,8 +700,10 @@ void do_insertfile(
     filestruct *edittop_save = openfile->edittop;
     size_t current_x_save = openfile->current_x;
     ssize_t current_y_save = openfile->current_y;
-    bool at_edittop = FALSE;
-	/* Whether we're at the top of the edit window. */
+    bool edittop_inside = FALSE;
+#ifndef NANO_TINY
+    bool right_side_up = FALSE, single_line = FALSE;
+#endif
 
     while (TRUE) {
 #ifndef NANO_TINY
@@ -741,7 +748,7 @@ void do_insertfile(
 	 * blank, open a new buffer instead of canceling.  If the
 	 * filename or command begins with a newline (i.e. an encoded
 	 * null), treat it as though it's blank. */
-	if (i == -1 || ((i == -2 || answer[0] == '\n')
+	if (i == -1 || ((i == -2 || *answer == '\n')
 #ifdef ENABLE_MULTIBUFFER
 		&& !ISSET(MULTIBUFFER)
 #endif
@@ -795,19 +802,34 @@ void do_insertfile(
 		)
 		continue;
 
+#ifndef NANO_TINY
+		/* Keep track of whether the mark begins inside the
+		 * partition and will need adjustment. */
+		if (openfile->mark_set) {
+		    filestruct *top, *bot;
+		    size_t top_x, bot_x;
+
+		    mark_order((const filestruct **)&top, &top_x,
+			(const filestruct **)&bot, &bot_x,
+			&right_side_up);
+
+		    single_line = (top == bot);
+		}
+#endif
+
 #ifdef ENABLE_MULTIBUFFER
 	    if (!ISSET(MULTIBUFFER)) {
 #endif
 		/* If we're not inserting into a new buffer, partition
 		 * the filestruct so that it contains no text and hence
 		 * looks like a new buffer, and keep track of whether
-		 * the top of the partition is the top of the edit
-		 * window. */
+		 * the top of the edit window is inside the
+		 * partition. */
 		filepart = partition_filestruct(openfile->current,
 			openfile->current_x, openfile->current,
 			openfile->current_x);
-		at_edittop =
-			(openfile->fileage == openfile->edittop);
+		edittop_inside =
+			(openfile->edittop == openfile->fileage);
 #ifdef ENABLE_MULTIBUFFER
 	    }
 #endif
@@ -864,14 +886,38 @@ void do_insertfile(
 		/* If we were at the top of the edit window before, set
 		 * the saved value of edittop to the new top of the edit
 		 * window. */
-		if (at_edittop)
+		if (edittop_inside)
 		    edittop_save = openfile->fileage;
 
 		/* Update the current x-coordinate to account for the
-		 * number of characters inserted on the current line. */
+		 * number of characters inserted on the current line.
+		 * If the mark begins inside the partition, adjust the
+		 * mark coordinates to compensate for the change in the
+		 * current line. */
 		openfile->current_x = strlen(openfile->filebot->data);
-		if (openfile->fileage == openfile->filebot)
+		if (openfile->fileage == openfile->filebot) {
+#ifndef NANO_TINY
+		    if (openfile->mark_set) {
+			openfile->mark_begin = openfile->current;
+			if (!right_side_up)
+			    openfile->mark_begin_x +=
+				openfile->current_x;
+		    }
+#endif
 		    openfile->current_x += current_x_save;
+		}
+#ifndef NANO_TINY
+		else if (openfile->mark_set) {
+		    if (!right_side_up) {
+			if (single_line) {
+			    openfile->mark_begin = openfile->current;
+			    openfile->mark_begin_x -= current_x_save;
+			} else
+			    openfile->mark_begin_x -=
+				openfile->current_x;
+		    }
+		}
+#endif
 
 		/* Update the current y-coordinate to account for the
 		 * number of lines inserted. */
@@ -1234,11 +1280,11 @@ void init_backup_dir(void)
  * on write error. */
 int copy_file(FILE *inn, FILE *out)
 {
+    int retval = 0;
     char buf[BUFSIZ];
     size_t charsread;
-    int retval = 0;
 
-    assert(inn != NULL && out != NULL);
+    assert(inn != NULL && out != NULL && inn != out);
 
     do {
 	charsread = fread(buf, sizeof(char), BUFSIZ, inn);
@@ -1307,7 +1353,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 
     assert(name != NULL);
 
-    if (name[0] == '\0')
+    if (*name == '\0')
 	return -1;
 
     if (f_open != NULL)
@@ -1408,7 +1454,7 @@ bool write_file(const char *name, FILE *f_open, bool tmp, append_type
 	    sprintf(backupname, "%s%s", backup_dir, backuptemp);
 	    free(backuptemp);
 	    backuptemp = get_next_filename(backupname, "~");
-	    if (backuptemp[0] == '\0') {
+	    if (*backuptemp == '\0') {
 		statusbar(_("Error writing %s: %s"), backupname,
 		    _("Too many backup files?"));
 		free(backuptemp);
@@ -1833,7 +1879,7 @@ bool do_writeout(bool exiting)
 
 	/* If the filename or command begins with a newline (i.e. an
 	 * encoded null), treat it as though it's blank. */
-	if (i < 0 || answer[0] == '\n') {
+	if (i < 0 || *answer == '\n') {
 	    statusbar(_("Cancelled"));
 	    retval = FALSE;
 	    break;
@@ -1993,7 +2039,7 @@ char *real_dir_from_tilde(const char *buf)
 
     assert(buf != NULL);
 
-    if (buf[0] == '~') {
+    if (*buf == '~') {
 	size_t i = 1;
 	char *tilde_dir;
 
@@ -2088,11 +2134,11 @@ bool is_dir(const char *buf)
 /* These functions, username_tab_completion(), cwd_tab_completion()
  * (originally exe_n_cwd_tab_completion()), and input_tab(), were
  * adapted from busybox 0.46 (cmdedit.c).  Here is the notice from that
- * file:
+ * file, with the copyright years updated:
  *
  * Termios command line History and Editting, originally
  * intended for NetBSD sh (ash)
- * Copyright (c) 1999
+ * Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
  *      Main code:            Adam Rogoyski <rogoyski@cs.utexas.edu>
  *      Etc:                  Dave Cinege <dcinege@psychosis.com>
  *  Majorly adjusted/re-written for busybox:
@@ -2103,20 +2149,20 @@ bool is_dir(const char *buf)
  * This code is 'as is' with no warranty.
  * This code may safely be consumed by a BSD or GPL license. */
 
-/* We consider the first buflen characters of buf for ~username tab
+/* We consider the first buf_len characters of buf for ~username tab
  * completion. */
 char **username_tab_completion(const char *buf, size_t *num_matches,
-	size_t buflen)
+	size_t buf_len)
 {
     char **matches = NULL;
     const struct passwd *userdata;
 
-    assert(buf != NULL && num_matches != NULL && buflen > 0);
+    assert(buf != NULL && num_matches != NULL && buf_len > 0);
 
     *num_matches = 0;
 
     while ((userdata = getpwent()) != NULL) {
-	if (strncmp(userdata->pw_name, buf + 1, buflen - 1) == 0) {
+	if (strncmp(userdata->pw_name, buf + 1, buf_len - 1) == 0) {
 	    /* Cool, found a match.  Add it to the list.  This makes a
 	     * lot more sense to me (Chris) this way... */
 
@@ -2140,10 +2186,10 @@ char **username_tab_completion(const char *buf, size_t *num_matches,
     return matches;
 }
 
-/* We consider the first buflen characters of buf for filename tab
+/* We consider the first buf_len characters of buf for filename tab
  * completion. */
 char **cwd_tab_completion(const char *buf, bool allow_files, size_t
-	*num_matches, size_t buflen)
+	*num_matches, size_t buf_len)
 {
     char *dirname = mallocstrcpy(NULL, buf), *filename;
 #ifndef DISABLE_OPERATINGDIR
@@ -2154,10 +2200,10 @@ char **cwd_tab_completion(const char *buf, bool allow_files, size_t
     DIR *dir;
     const struct dirent *nextdir;
 
-    assert(dirname != NULL && num_matches != NULL && buflen >= 0);
+    assert(dirname != NULL && num_matches != NULL);
 
     *num_matches = 0;
-    null_at(&dirname, buflen);
+    null_at(&dirname, buf_len);
 
     /* Okie, if there's a / in the buffer, strip out the directory
      * part. */
@@ -2246,7 +2292,7 @@ char **cwd_tab_completion(const char *buf, bool allow_files, size_t
 char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	*lastwastab, void (*refresh_func)(void), bool *list)
 {
-    size_t num_matches = 0;
+    size_t num_matches = 0, buf_len;
     char **matches = NULL;
 
     assert(buf != NULL && place != NULL && *place <= strlen(buf) && lastwastab != NULL && refresh_func != NULL && list != NULL);
@@ -2255,7 +2301,7 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 
     /* If the word starts with `~' and there is no slash in the word,
      * then try completing this word as a username. */
-    if (*place > 0 && buf[0] == '~') {
+    if (*place > 0 && *buf == '~') {
 	const char *bob = strchr(buf, '/');
 
 	if (bob == NULL || bob >= buf + *place)
@@ -2268,7 +2314,9 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	matches = cwd_tab_completion(buf, allow_files, &num_matches,
 		*place);
 
-    if (num_matches == 0)
+    buf_len = strlen(buf);
+
+    if (num_matches == 0 || *place != buf_len)
 	beep();
     else {
 	size_t match, common_len = 0;
@@ -2327,8 +2375,6 @@ char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	 * twice in succession with no statusbar changes to see a match
 	 * list. */
 	if (common_len != *place) {
-	    size_t buf_len = strlen(buf);
-
 	    *lastwastab = FALSE;
 	    buf = charealloc(buf, common_len + buf_len - *place + 1);
 	    charmove(buf + common_len, buf + *place, buf_len -
@@ -2475,10 +2521,10 @@ void load_history(void)
 	     * Assume the last history entry is a blank line. */
 	    filestruct **history = &search_history;
 	    char *line = NULL;
-	    size_t buflen = 0;
+	    size_t buf_len = 0;
 	    ssize_t read;
 
-	    while ((read = getline(&line, &buflen, hist)) >= 0) {
+	    while ((read = getline(&line, &buf_len, hist)) >= 0) {
 		if (read > 0 && line[read - 1] == '\n') {
 		    read--;
 		    line[read] = '\0';
