@@ -1,12 +1,12 @@
-/* $Id: nano.c,v 1.582 2007/01/12 02:58:12 dolorous Exp $ */
+/* $Id: nano.c,v 1.582.2.23 2007/12/18 22:00:17 dolorous Exp $ */
 /**************************************************************************
  *   nano.c                                                               *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -40,6 +40,10 @@
 #include <sys/ioctl.h>
 #endif
 
+#ifndef DISABLE_MOUSE
+static int oldinterval = -1;
+	/* Used to store the user's original mouse click interval. */
+#endif
 #ifdef ENABLE_NANORC
 static bool no_rcfiles = FALSE;
 	/* Should we ignore all rcfiles? */
@@ -343,22 +347,23 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 	    renumber(file_bot_save->next);
     }
 
-    /* Since the text has now been saved, remove it from the filestruct.
-     * If the mark begins inside the partition, set the beginning of the
-     * mark to where the saved text used to start. */
+    /* Since the text has now been saved, remove it from the
+     * filestruct. */
     openfile->fileage = (filestruct *)nmalloc(sizeof(filestruct));
     openfile->fileage->data = mallocstrcpy(NULL, "");
     openfile->filebot = openfile->fileage;
-#ifndef NANO_TINY
-    if (mark_inside) {
-	openfile->mark_begin = openfile->fileage;
-	openfile->mark_begin_x = top_x;
-    }
-#endif
 
-    /* Restore the current line and cursor position. */
+    /* Restore the current line and cursor position.  If the mark begins
+     * inside the partition, set the beginning of the mark to where the
+     * saved text used to start. */
     openfile->current = openfile->fileage;
     openfile->current_x = top_x;
+#ifndef NANO_TINY
+    if (mark_inside) {
+	openfile->mark_begin = openfile->current;
+	openfile->mark_begin_x = openfile->current_x;
+    }
+#endif
 
     top_save = openfile->fileage;
 
@@ -391,10 +396,28 @@ void move_to_filestruct(filestruct **file_top, filestruct **file_bot,
 void copy_from_filestruct(filestruct *file_top, filestruct *file_bot)
 {
     filestruct *top_save;
+    size_t current_x_save = openfile->current_x;
     bool edittop_inside;
+#ifndef NANO_TINY
+    bool right_side_up = FALSE, single_line = FALSE;
+#endif
 
     assert(file_top != NULL && file_bot != NULL);
 
+#ifndef NANO_TINY
+    /* Keep track of whether the mark begins inside the partition and
+     * will need adjustment. */
+    if (openfile->mark_set) {
+	filestruct *top, *bot;
+	size_t top_x, bot_x;
+
+	mark_order((const filestruct **)&top, &top_x,
+		(const filestruct **)&bot, &bot_x, &right_side_up);
+
+	single_line = (top == bot);
+    }
+#endif
+
     /* Partition the filestruct so that it contains no text, and keep
      * track of whether the top of the edit window is inside the
      * partition. */
@@ -409,11 +432,32 @@ void copy_from_filestruct(filestruct *file_top, filestruct *file_bot)
     while (openfile->filebot->next != NULL)
 	openfile->filebot = openfile->filebot->next;
 
-    /* Restore the current line and cursor position. */
+    /* Restore the current line and cursor position.  If the mark begins
+     * inside the partition, adjust the mark coordinates to compensate
+     * for the change in the current line. */
     openfile->current = openfile->filebot;
     openfile->current_x = strlen(openfile->filebot->data);
-    if (openfile->fileage == openfile->filebot)
-	openfile->current_x += strlen(filepart->top_data);
+    if (openfile->fileage == openfile->filebot) {
+#ifndef NANO_TINY
+	if (openfile->mark_set) {
+	    openfile->mark_begin = openfile->current;
+	    if (!right_side_up)
+		openfile->mark_begin_x += openfile->current_x;
+	}
+#endif
+	openfile->current_x += current_x_save;
+    }
+#ifndef NANO_TINY
+    else if (openfile->mark_set) {
+	if (!right_side_up) {
+	    if (single_line) {
+		openfile->mark_begin = openfile->current;
+		openfile->mark_begin_x -= current_x_save;
+	    } else
+		openfile->mark_begin_x -= openfile->current_x;
+	}
+    }
+#endif
 
     /* Get the number of characters in the copied text, and add it to
      * totsize. */
@@ -608,7 +652,7 @@ void die_save_file(const char *die_filename)
 
     /* If we can't save, we have really bad problems, but we might as
      * well try. */
-    if (die_filename[0] == '\0')
+    if (*die_filename == '\0')
 	die_filename = "nano";
 
     retval = get_next_filename(die_filename, ".save");
@@ -670,13 +714,14 @@ void window_init(void)
 void disable_mouse_support(void)
 {
     mousemask(0, NULL);
+    mouseinterval(oldinterval);
 }
 
 /* Enable mouse support. */
 void enable_mouse_support(void)
 {
     mousemask(ALL_MOUSE_EVENTS, NULL);
-    mouseinterval(50);
+    oldinterval = mouseinterval(50);
 }
 
 /* Initialize mouse support.  Enable it if the USE_MOUSE flag is set,
@@ -706,14 +751,14 @@ void print_opt_full(const char *shortflag
 	, const char *desc)
 {
     printf(" %s\t", shortflag);
-    if (strlen(shortflag) < 8)
+    if (strlenpt(shortflag) < 8)
 	printf("\t");
 
 #ifdef HAVE_GETOPT_LONG
     printf("%s\t", longflag);
-    if (strlen(longflag) < 8)
+    if (strlenpt(longflag) < 8)
 	printf("\t\t");
-    else if (strlen(longflag) < 16)
+    else if (strlenpt(longflag) < 16)
 	printf("\t");
 #endif
 
@@ -841,6 +886,8 @@ void version(void)
 {
     printf(_(" GNU nano version %s (compiled %s, %s)\n"), VERSION,
 	__TIME__, __DATE__);
+    printf(" (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007\n");
+    printf(" Free Software Foundation, Inc.\n");
     printf(
 	_(" Email: nano@nano-editor.org	Web: http://www.nano-editor.org/"));
     printf(_("\n Compiled options:"));
@@ -1209,6 +1256,7 @@ void disable_extended_io(void)
     tcsetattr(0, TCSANOW, &term);
 }
 
+#ifdef USE_SLANG
 /* Disable interpretation of the special control keys in our terminal
  * settings. */
 void disable_signals(void)
@@ -1219,6 +1267,7 @@ void disable_signals(void)
     term.c_lflag &= ~ISIG;
     tcsetattr(0, TCSANOW, &term);
 }
+#endif
 
 #ifndef NANO_TINY
 /* Enable interpretation of the special control keys in our terminal
@@ -1255,37 +1304,41 @@ void enable_flow_control(void)
     tcsetattr(0, TCSANOW, &term);
 }
 
-/* Set up the terminal state.  Put the terminal in cbreak mode (read one
- * character at a time and interpret the special control keys), disable
- * translation of carriage return (^M) into newline (^J) so that we can
- * tell the difference between the Enter key and Ctrl-J, and disable
- * echoing of characters as they're typed.  Finally, disable extended
- * input and output processing, disable interpretation of the special
- * control keys, and if we're not in preserve mode, disable
- * interpretation of the flow control characters too. */
+/* Set up the terminal state.  Put the terminal in raw mode (read one
+ * character at a time, disable the special control keys, and disable
+ * the flow control characters), disable translation of carriage return
+ * (^M) into newline (^J) so that we can tell the difference between the
+ * Enter key and Ctrl-J, and disable echoing of characters as they're
+ * typed.  Finally, disable extended input and output processing, and,
+ * if we're not in preserve mode, reenable interpretation of the flow
+ * control characters. */
 void terminal_init(void)
 {
 #ifdef USE_SLANG
     /* Slang curses emulation brain damage, part 2: Slang doesn't
-     * implement nonl() or noecho() properly, so there's no way to
-     * properly reinitialize the terminal using them.  We have to save
-     * the terminal state after the first call and restore it on
-     * subsequent calls. */
+     * implement raw(), nonl(), or noecho() properly, so there's no way
+     * to properly reinitialize the terminal using them.  We have to
+     * disable the special control keys and interpretation of the flow
+     * control characters using termios, save the terminal state after
+     * the first call, and restore it on subsequent calls. */
     static struct termios newterm;
     static bool newterm_set = FALSE;
 
     if (!newterm_set) {
 #endif
 
-	cbreak();
+	raw();
 	nonl();
 	noecho();
 	disable_extended_io();
+	if (ISSET(PRESERVE))
+	    enable_flow_control();
+
+#ifdef USE_SLANG
 	disable_signals();
 	if (!ISSET(PRESERVE))
 	    disable_flow_control();
 
-#ifdef USE_SLANG
 	tcgetattr(0, &newterm);
 	newterm_set = TRUE;
     } else
@@ -1332,7 +1385,7 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	/* If we got a mouse click and it was on a shortcut, read in the
 	 * shortcut character. */
 	if (*func_key && input == KEY_MOUSE) {
-	    if (do_mouse())
+	    if (do_mouse() == 1)
 		input = get_kbinput(edit, meta_key, func_key);
 	    else {
 		*meta_key = FALSE;
@@ -1490,50 +1543,43 @@ int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 
 #ifndef DISABLE_MOUSE
 /* Handle a mouse click on the edit window or the shortcut list. */
-bool do_mouse(void)
+int do_mouse(void)
 {
     int mouse_x, mouse_y;
-    bool retval = get_mouseinput(&mouse_x, &mouse_y, TRUE);
+    int retval = get_mouseinput(&mouse_x, &mouse_y, TRUE);
 
-    if (!retval) {
-	/* We can click in the edit window to move the cursor. */
-	if (wenclose(edit, mouse_y, mouse_x)) {
-	    bool sameline;
-		/* Did they click on the line with the cursor?  If they
-		 * clicked on the cursor, we set the mark. */
-	    const filestruct *current_save = openfile->current;
-	    size_t current_x_save = openfile->current_x;
-	    size_t pww_save = openfile->placewewant;
+    /* We can click on the edit window to move the cursor. */
+    if (retval == 0 && wmouse_trafo(edit, &mouse_y, &mouse_x, FALSE)) {
+	bool sameline;
+	    /* Did they click on the line with the cursor?  If they
+	     * clicked on the cursor, we set the mark. */
+	const filestruct *current_save = openfile->current;
+	size_t current_x_save = openfile->current_x;
+	size_t pww_save = openfile->placewewant;
 
-	    /* Subtract out the size of topwin. */
-	    mouse_y -= 2 - no_more_space();
+	sameline = (mouse_y == openfile->current_y);
 
-	    sameline = (mouse_y == openfile->current_y);
+	/* Move to where the click occurred. */
+	for (; openfile->current_y < mouse_y && openfile->current !=
+		openfile->filebot; openfile->current_y++)
+	    openfile->current = openfile->current->next;
+	for (; openfile->current_y > mouse_y && openfile->current !=
+		openfile->fileage; openfile->current_y--)
+	    openfile->current = openfile->current->prev;
 
-	    /* Move to where the click occurred. */
-	    for (; openfile->current_y < mouse_y &&
-		openfile->current != openfile->filebot;
-		openfile->current_y++)
-		openfile->current = openfile->current->next;
-	    for (; openfile->current_y > mouse_y &&
-		openfile->current != openfile->fileage;
-		openfile->current_y--)
-		openfile->current = openfile->current->prev;
-
-	    openfile->current_x = actual_x(openfile->current->data,
+	openfile->current_x = actual_x(openfile->current->data,
 		get_page_start(xplustabs()) + mouse_x);
-	    openfile->placewewant = xplustabs();
+	openfile->placewewant = xplustabs();
 
 #ifndef NANO_TINY
-	    /* Clicking where the cursor is toggles the mark, as does
-	     * clicking beyond the line length with the cursor at the
-	     * end of the line. */
-	    if (sameline && openfile->current_x == current_x_save)
-		do_mark();
+	/* Clicking where the cursor is toggles the mark, as does
+	 * clicking beyond the line length with the cursor at the end of
+	 * the line. */
+	if (sameline && openfile->current_x == current_x_save)
+	    do_mark();
 #endif
 
-	    edit_redraw(current_save, pww_save);
-	}
+	edit_redraw(current_save, pww_save);
     }
 
     return retval;
@@ -1996,7 +2042,7 @@ int main(int argc, char **argv)
 	    free(backup_dir);
 	    backup_dir = backup_dir_cpy;
 	}
-#endif	
+#endif
 #ifndef DISABLE_JUSTIFY
 	if (quotestr_cpy != NULL) {
 	    free(quotestr);
@@ -2122,8 +2168,9 @@ int main(int argc, char **argv)
     /* Back up the old terminal settings so that they can be restored. */
     tcgetattr(0, &oldterm);
 
-    /* Initialize curses mode. */
-    initscr();
+    /* Initialize curses mode.  If this fails, get out. */
+    if (initscr() == NULL)
+	exit(1);
 
     /* Set up the terminal state. */
     terminal_init();
@@ -2223,13 +2270,14 @@ int main(int argc, char **argv)
 
     display_main_list();
 
-    display_buffer();    
+    display_buffer();
 
     while (TRUE) {
 	bool meta_key, func_key, s_or_t, ran_func, finished;
 
 	/* Make sure the cursor is in the edit window. */
 	reset_cursor();
+	wnoutrefresh(edit);
 
 #ifndef NANO_TINY
 	if (!jump_buf_main) {
