@@ -1,12 +1,12 @@
-/* $Id: prompt.c,v 1.75.2.1 2007/04/21 18:53:38 dolorous Exp $ */
+/* $Id: prompt.c,v 1.75.2.10 2007/12/08 07:00:27 dolorous Exp $ */
 /**************************************************************************
  *   prompt.c                                                             *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -76,7 +76,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 	/* If we got a mouse click and it was on a shortcut, read in the
 	 * shortcut character. */
 	if (*func_key && input == KEY_MOUSE) {
-	    if (do_statusbar_mouse())
+	    if (do_statusbar_mouse() == 1)
 		input = get_kbinput(bottomwin, meta_key, func_key);
 	    else {
 		*meta_key = FALSE;
@@ -273,37 +273,31 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 
 #ifndef DISABLE_MOUSE
 /* Handle a mouse click on the statusbar prompt or the shortcut list. */
-bool do_statusbar_mouse(void)
+int do_statusbar_mouse(void)
 {
     int mouse_x, mouse_y;
-    bool retval = get_mouseinput(&mouse_x, &mouse_y, TRUE);
+    int retval = get_mouseinput(&mouse_x, &mouse_y, TRUE);
 
-    if (!retval) {
-	/* We can click in the statusbar window text to move the
-	 * cursor. */
-	if (wenclose(bottomwin, mouse_y, mouse_x)) {
-	    size_t start_col;
+    /* We can click on the statusbar window text to move the cursor. */
+    if (retval == 0 && wmouse_trafo(bottomwin, &mouse_y, &mouse_x,
+	FALSE)) {
+	size_t start_col;
 
-	    assert(prompt != NULL);
+	assert(prompt != NULL);
 
-	    start_col = strlenpt(prompt) + 1;
+	start_col = strlenpt(prompt) + 2;
 
-	    /* Subtract out the sizes of topwin and edit. */
-	    mouse_y -= (2 - no_more_space()) + editwinrows;
+	/* Move to where the click occurred. */
+	if (mouse_x >= start_col && mouse_y == 0) {
+	    size_t pww_save = statusbar_pww;
 
-	    /* Move to where the click occurred. */
-	    if (mouse_x > start_col && mouse_y == 0) {
-		size_t pww_save = statusbar_pww;
-
-		statusbar_x = actual_x(answer,
+	    statusbar_x = actual_x(answer,
 			get_statusbar_page_start(start_col, start_col +
-			statusbar_xplustabs()) + mouse_x - start_col -
-			1);
-		statusbar_pww = statusbar_xplustabs();
+			statusbar_xplustabs()) + mouse_x - start_col);
+	    statusbar_pww = statusbar_xplustabs();
 
-		if (need_statusbar_horizontal_update(pww_save))
-		    update_statusbar_line(answer, statusbar_x);
-	    }
+	    if (need_statusbar_horizontal_update(pww_save))
+		update_statusbar_line(answer, statusbar_x);
 	}
     }
 
@@ -870,10 +864,10 @@ size_t get_statusbar_page_start(size_t start_col, size_t column)
 /* Put the cursor in the statusbar prompt at statusbar_x. */
 void reset_statusbar_cursor(void)
 {
-    size_t start_col = strlenpt(prompt) + 1;
+    size_t start_col = strlenpt(prompt) + 2;
     size_t xpt = statusbar_xplustabs();
 
-    wmove(bottomwin, 0, start_col + 1 + xpt -
+    wmove(bottomwin, 0, start_col + xpt -
 	get_statusbar_page_start(start_col, start_col + xpt));
 }
 
@@ -887,7 +881,7 @@ void update_statusbar_line(const char *curranswer, size_t index)
 
     assert(prompt != NULL && index <= strlen(curranswer));
 
-    start_col = strlenpt(prompt) + 1;
+    start_col = strlenpt(prompt) + 2;
     index = strnlenpt(curranswer, index);
     page_start = get_statusbar_page_start(start_col, start_col + index);
 
@@ -904,10 +898,9 @@ void update_statusbar_line(const char *curranswer, size_t index)
     waddstr(bottomwin, expanded);
     free(expanded);
 
-    reset_statusbar_cursor();
-
     wattroff(bottomwin, reverse_attr);
 
+    reset_statusbar_cursor();
     wnoutrefresh(bottomwin);
 }
 
@@ -916,7 +909,7 @@ void update_statusbar_line(const char *curranswer, size_t index)
  * different pages. */
 bool need_statusbar_horizontal_update(size_t pww_save)
 {
-    size_t start_col = strlenpt(prompt) + 1;
+    size_t start_col = strlenpt(prompt) + 2;
 
     return get_statusbar_page_start(start_col, start_col + pww_save) !=
 	get_statusbar_page_start(start_col, start_col + statusbar_pww);
@@ -1082,7 +1075,7 @@ int get_prompt_string(bool allow_tabs,
 		     * magichistory is set, save magichistory in
 		     * answer. */
 		    if ((*history_list)->next == NULL &&
-			answer[0] == '\0' && magichistory != NULL) {
+			*answer == '\0' && magichistory != NULL) {
 			answer = mallocstrcpy(answer, magichistory);
 			statusbar_x = strlen(answer);
 		    }
@@ -1122,6 +1115,7 @@ int get_prompt_string(bool allow_tabs,
 #endif
 
 	reset_statusbar_cursor();
+	wnoutrefresh(bottomwin);
     }
 
 #ifndef NANO_TINY
@@ -1226,7 +1220,7 @@ int do_prompt(bool allow_tabs,
 	    retval = -1;
 	    break;
 	case NANO_ENTER_KEY:
-	    retval = (answer[0] == '\0') ? -2 : 0;
+	    retval = (*answer == '\0') ? -2 : 0;
 	    break;
     }
 
@@ -1337,18 +1331,17 @@ int do_yesno_prompt(bool all, const char *msg)
 		break;
 #ifndef DISABLE_MOUSE
 	    case KEY_MOUSE:
-		get_mouseinput(&mouse_x, &mouse_y, FALSE);
-
-		if (wenclose(bottomwin, mouse_y, mouse_x) &&
-			!ISSET(NO_HELP) && mouse_x < (width * 2) &&
-			mouse_y - (2 - no_more_space()) -
-			editwinrows - 1 >= 0) {
+		/* We can click on the Yes/No/All shortcut list to
+		 * select an answer. */
+		if (get_mouseinput(&mouse_x, &mouse_y, FALSE) == 0 &&
+			wmouse_trafo(bottomwin, &mouse_y, &mouse_x,
+			FALSE) && mouse_x < (width * 2) &&
+			mouse_y > 0) {
 		    int x = mouse_x / width;
 			/* Calculate the x-coordinate relative to the
 			 * two columns of the Yes/No/All shortcuts in
 			 * bottomwin. */
-		    int y = mouse_y - (2 - no_more_space()) -
-			editwinrows - 1;
+		    int y = mouse_y - 1;
 			/* Calculate the y-coordinate relative to the
 			 * beginning of the Yes/No/All shortcuts in
 			 * bottomwin, i.e. with the sizes of topwin,
