@@ -1,12 +1,12 @@
-/* $Id: proto.h,v 1.372 2007/01/11 21:36:29 dolorous Exp $ */
+/* $Id: proto.h,v 1.372.2.8 2007/12/18 15:55:48 dolorous Exp $ */
 /**************************************************************************
  *   proto.h                                                              *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -203,18 +203,18 @@ int nstrncasecmp(const char *s1, const char *s2, size_t n);
 #endif
 int mbstrncasecmp(const char *s1, const char *s2, size_t n);
 #ifndef HAVE_STRCASESTR
-const char *nstrcasestr(const char *haystack, const char *needle);
+char *nstrcasestr(const char *haystack, const char *needle);
 #endif
-const char *mbstrcasestr(const char *haystack, const char *needle);
+char *mbstrcasestr(const char *haystack, const char *needle);
 #if !defined(NANO_TINY) || !defined(DISABLE_TABCOMP)
-const char *revstrstr(const char *haystack, const char *needle, const
-	char *rev_start);
+char *revstrstr(const char *haystack, const char *needle, const char
+	*rev_start);
 #endif
 #ifndef NANO_TINY
-const char *revstrcasestr(const char *haystack, const char *needle,
-	const char *rev_start);
-const char *mbrevstrcasestr(const char *haystack, const char *needle,
-	const char *rev_start);
+char *revstrcasestr(const char *haystack, const char *needle, const char
+	*rev_start);
+char *mbrevstrcasestr(const char *haystack, const char *needle, const
+	char *rev_start);
 #endif
 size_t mbstrlen(const char *s);
 #ifndef HAVE_STRNLEN
@@ -326,9 +326,9 @@ void free_chararray(char **array, size_t len);
 #ifndef DISABLE_TABCOMP
 bool is_dir(const char *buf);
 char **username_tab_completion(const char *buf, size_t *num_matches,
-	size_t buflen);
+	size_t buf_len);
 char **cwd_tab_completion(const char *buf, bool allow_files, size_t
-	*num_matches, size_t buflen);
+	*num_matches, size_t buf_len);
 char *input_tab(char *buf, bool allow_files, size_t *place, bool
 	*lastwastab, void (*refresh_func)(void), bool *list);
 #endif
@@ -475,7 +475,9 @@ void allow_pending_sigwinch(bool allow);
 void do_toggle(const toggle *which);
 #endif
 void disable_extended_io(void);
+#ifdef USE_SLANG
 void disable_signals(void);
+#endif
 #ifndef NANO_TINY
 void enable_signals(void);
 #endif
@@ -485,7 +487,7 @@ void terminal_init(void);
 int do_input(bool *meta_key, bool *func_key, bool *s_or_t, bool
 	*ran_func, bool *finished, bool allow_funcs);
 #ifndef DISABLE_MOUSE
-bool do_mouse(void);
+int do_mouse(void);
 #endif
 void do_output(char *output, size_t output_len, bool allow_cntrls);
 
@@ -494,7 +496,7 @@ int do_statusbar_input(bool *meta_key, bool *func_key, bool *s_or_t,
 	bool *ran_func, bool *finished, bool allow_funcs, void
 	(*refresh_func)(void));
 #ifndef DISABLE_MOUSE
-bool do_statusbar_mouse(void);
+int do_statusbar_mouse(void);
 #endif
 void do_statusbar_output(char *output, size_t output_len, bool
 	*got_enter, bool allow_cntrls);
@@ -749,7 +751,7 @@ void unparse_kbinput(char *output, size_t output_len);
 int *get_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
 int *parse_verbatim_kbinput(WINDOW *win, size_t *kbinput_len);
 #ifndef DISABLE_MOUSE
-bool get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
+int get_mouseinput(int *mouse_x, int *mouse_y, bool allow_shortcuts);
 #endif
 const shortcut *get_shortcut(const shortcut *s_list, int *kbinput, bool
 	*meta_key, bool *func_key);
