@@ -1,12 +1,12 @@
-/* $Id: search.c,v 1.197.2.1 2007/04/19 03:15:04 dolorous Exp $ */
+/* $Id: search.c,v 1.197.2.6 2007/10/11 05:01:31 dolorous Exp $ */
 /**************************************************************************
  *   search.c                                                             *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -208,8 +208,8 @@ int search_init(bool replacing, bool use_answer)
     backupstring = NULL;
 
     /* Cancel any search, or just return with no previous search. */
-    if (i == -1 || (i < 0 && last_search[0] == '\0') ||
-	    (!replacing && i == 0 && answer[0] == '\0')) {
+    if (i == -1 || (i < 0 && *last_search == '\0') || (!replacing &&
+	i == 0 && *answer == '\0')) {
 	statusbar(_("Cancelled"));
 	return -1;
     } else {
@@ -444,7 +444,7 @@ void do_search(void)
 	return;
 
     /* If answer is now "", copy last_search into answer. */
-    if (answer[0] == '\0')
+    if (*answer == '\0')
 	answer = mallocstrcpy(answer, last_search);
     else
 	last_search = mallocstrcpy(last_search, answer);
@@ -827,7 +827,8 @@ ssize_t do_replace_loop(
 		openfile->current_x += match_len + length_change - 1;
 
 	    /* Cleanup. */
-	    openfile->totsize += length_change;
+	    openfile->totsize += mbstrlen(copy) -
+		mbstrlen(openfile->current->data);
 	    free(openfile->current->data);
 	    openfile->current->data = copy;
 
