@@ -1,12 +1,12 @@
-/* $Id: text.c,v 1.163 2007/01/12 02:58:12 dolorous Exp $ */
+/* $Id: text.c,v 1.163.2.5 2007/12/18 01:28:53 dolorous Exp $ */
 /**************************************************************************
  *   text.c                                                               *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -503,9 +503,10 @@ bool execute_command(const char *command)
     if (!sig_failed && sigaction(SIGINT, &oldaction, NULL) == -1)
 	nperror("sigaction");
 
-    /* Disable interpretation of the special control keys so that we can
+    /* Restore the terminal to its previous state.  In the process,
+     * disable interpretation of the special control keys so that we can
      * use Ctrl-C for other things. */
-    disable_signals();
+    terminal_init();
 
     return TRUE;
 }
@@ -1212,9 +1213,16 @@ void backup_lines(filestruct *first_line, size_t par_len)
      * line, putting first_line, edittop, current, and mark_begin at the
      * same lines in the copied paragraph that they had in the original
      * paragraph. */
-    if (openfile->current != openfile->fileage)
+    if (openfile->current != openfile->fileage) {
 	top = openfile->current->prev;
-    else
+#ifndef NANO_TINY
+	if (old_mark_set &&
+		openfile->current->lineno == mb_lineno_save) {
+	    openfile->mark_begin = openfile->current;
+	    openfile->mark_begin_x = mark_begin_x_save;
+	}
+#endif
+    } else
 	top = openfile->current;
     for (i = par_len; i > 0 && top != NULL; i--) {
 	if (top->lineno == fl_lineno_save)
