@@ -1,12 +1,12 @@
-/* $Id: utils.c,v 1.124 2007/01/01 05:15:32 dolorous Exp $ */
+/* $Id: utils.c,v 1.124.2.6 2007/10/11 05:01:32 dolorous Exp $ */
 /**************************************************************************
  *   utils.c                                                              *
  *                                                                        *
- *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Chris Allegretta    *
- *   Copyright (C) 2005, 2006, 2007 David Lawrence Ramsey                 *
+ *   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007   *
+ *   Free Software Foundation, Inc.                                       *
  *   This program is free software; you can redistribute it and/or modify *
  *   it under the terms of the GNU General Public License as published by *
- *   the Free Software Foundation; either version 2, or (at your option)  *
+ *   the Free Software Foundation; either version 3, or (at your option)  *
  *   any later version.                                                   *
  *                                                                        *
  *   This program is distributed in the hope that it will be useful, but  *
@@ -162,14 +162,15 @@ void sunder(char *str)
  * (originally getdelim()), were adapted from GNU mailutils 0.5
  * (mailbox/getline.c).  Here is the notice from that file, after
  * converting to the GPL via LGPL clause 3, and with the Free Software
- * Foundation's address updated:
+ * Foundation's address and the copyright years updated:
  *
  * GNU Mailutils -- a suite of utilities for electronic mail
- * Copyright (C) 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+ * Copyright (C) 1999, 2000, 2001, 2002, 2004, 2005, 2006, 2007
+ * Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
+ * published by the Free Software Foundation; either version 3 of the
  * License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
@@ -207,15 +208,15 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 
     /* Allocate the line the first time. */
     if (*lineptr == NULL) {
-	*lineptr = charalloc(MAX_BUF_SIZE);
 	*n = MAX_BUF_SIZE;
+	*lineptr = charalloc(*n);
     }
 
     while ((c = getc(stream)) != EOF) {
 	/* Check if more memory is needed. */
 	if (indx >= *n) {
-	    *lineptr = charealloc(*lineptr, *n + MAX_BUF_SIZE);
 	    *n += MAX_BUF_SIZE;
+	    *lineptr = charealloc(*lineptr, *n);
 	}
 
 	/* Put the result in the line. */
@@ -228,8 +229,8 @@ ssize_t ngetdelim(char **lineptr, size_t *n, int delim, FILE *stream)
 
     /* Make room for the null character. */
     if (indx >= *n) {
-	*lineptr = charealloc(*lineptr, *n + MAX_BUF_SIZE);
 	*n += MAX_BUF_SIZE;
+	*lineptr = charealloc(*lineptr, *n);
     }
 
     /* Null-terminate the buffer. */
@@ -527,7 +528,7 @@ void remove_magicline(void)
 
 /* Set top_x and bot_x to the top and bottom x-coordinates of the mark,
  * respectively, based on the locations of top and bot.  If
- * right_side_up isn't NULL, set it to TRUE If the mark begins with
+ * right_side_up isn't NULL, set it to TRUE if the mark begins with
  * (mark_begin, mark_begin_x) and ends with (current, current_x), or
  * FALSE otherwise. */
 void mark_order(const filestruct **top, size_t *top_x, const filestruct
